@@ -1,10 +1,14 @@
 from ._mainAcoustic import AcousticField
 from .AcousticEnums import WaveType
 from AOT_biomaps.Config import config
+from .AcousticTools import next_power_of_2, reshape_field
+
 import ctypes
 import os
 import numpy as np
+from math import ceil
 import matplotlib.pyplot as plt
+from kwave.kgrid import kWaveGrid
 from kwave.ksource import kSource
 from kwave.ksensor import kSensor
 from kwave.kspaceFirstOrder3D import kspaceFirstOrder3D
@@ -43,53 +47,55 @@ class FocusedWave(AcousticField):
             print(f"Error generating file name: {e}")
             return None
 
-    def _apply_delay(self):
+    def _apply_delay(self, dx=None):
         """
-        Apply a temporal delay to focus the wave at a given lateral position (x_focal) and fixed focal depth (Foc).
+        Apply a temporal delay to focus the wave at a given focal point (self.focal_line, self.params['Foc']).
         Returns:
-            ndarray: Delayed signals, shape (nbPiezo, len(burst) + max_delay).
+            ndarray: Delayed signals, shape (total_grid_points, len(burst) + max_delay).
         """
         try:
-            # 1. Positions latérales de tous les éléments (en mètres)
-            element_positions = np.linspace(self.params['Xrange'][0], self.params['Xrange'][1], self.params['num_elements'])
+            if dx is None:
+                dx = self.params['dx']
 
-            # 2. Trouver l'indice de l'élément le plus proche de self.focal_line
-            center_idx = np.argmin(np.abs(element_positions - self.focal_line))
+            # 1. Calculate the total number of grid points for all elements
+            total_grid_points = self.params['num_elements'] * int(round(self.params['element_width'] / dx))
 
-            start_idx = max(0, center_idx - self.params['N_piezoFocal'] // 2)
-            end_idx = min(self.params['num_elements'] - 1, start_idx + self.params['N_piezoFocal'] - 1)
+            # 2. Calculate the physical positions of all grid points (in meters)
+            # The transducer is centered around 0 for simplicity, but adjust if needed
+            grid_positions = np.linspace(
+                self.params['Xrange'][0],
+                self.params['Xrange'][1] - dx,  # Subtract dx to avoid exceeding the range
+                total_grid_points
+            )
 
-            # 4. Positions des éléments sélectionnés (en mètres)
-            selected_indices = np.arange(start_idx, end_idx + 1)
-            selected_positions = element_positions[selected_indices]
+            # 3. Calculate distances from each grid point to the focal point (self.focal_line, self.params['Foc'])
+            distances = np.sqrt(
+                (grid_positions - self.focal_line)**2 +  # Lateral distance
+                self.params['Foc']**2                     # Axial distance (fixed)
+            )
 
-            # 5. Distance entre chaque élément sélectionné et le point focal (self.focal_line, self.params['Foc'])
-            distances = np.sqrt((self.focal_line - selected_positions)**2 + self.params['Foc']**2)
-
-            # 6. Délais en secondes : (distance_max - distance) / self.params['c0']
+            # 4. Calculate delays: (max_distance - distance) / c0
             max_distance = np.max(distances)
-            delays = (max_distance - distances) / self.params['c0']
+            delays = (max_distance - distances) / self.params['c0']  # Delays in seconds
 
-            # 7. Conversion en échantillons (vérifier que self.kgrid.dt est en secondes)
+            # 5. Convert delays to samples
             delay_samples = np.round(delays / self.kgrid.dt).astype(int)
             max_delay = np.max(delay_samples)
 
-            # 8. Application des délais
-            delayed_signals = np.zeros((self.params['num_elements'], len(self.burst) + max_delay))
-            for i, idx in enumerate(selected_indices):
+            # 6. Initialize the delayed signals array
+            delayed_signals = np.zeros((total_grid_points, len(self.burst) + max_delay))
+
+            # 7. Apply delays to the burst signal
+            for i in range(total_grid_points):
                 shift = delay_samples[i]
-                delayed_signals[idx, shift:shift + len(self.burst)] = self.burst
+                delayed_signals[i, shift:shift + len(self.burst)] = self.burst
 
             return delayed_signals
 
         except Exception as e:
             print(f"Error applying delay: {e}")
             return None
-
-
-
-
-        
+  
     def plot_delay(self):
         """
         Plot the time of the maximum of each delayed signal to visualize the wavefront.
@@ -114,67 +120,75 @@ class FocusedWave(AcousticField):
 
 
     def _generate_2Dacoustic_field_KWAVE(self, isGPU=True if config.get_process() == 'gpu' else False, show_log=True):
-        """
-        Generate a 2D acoustic field using k-Wave simulation for a focused wave.
-        Parameters:
-            - isGPU (bool): Flag indicating whether to use GPU for simulation.
-            - show_log (bool): Flag to show simulation logs.
-        Returns:
-            ndarray: Simulated acoustic field data.
-        """
         try:
-            # Get the active Conda environment path
-            conda_prefix = os.environ.get('CONDA_PREFIX', '')
-            if not conda_prefix:
-                raise RuntimeError("CONDA_PREFIX not set. Activate your Conda environment first.")
-
-            # Path to libsz.so.2 in the active environment
-            libsz_path = os.path.join(conda_prefix, 'lib', 'libsz.so.2')
-
-            # Add the Conda library path to LD_LIBRARY_PATH to ensure the subprocess can find libsz.so.2
-            if 'LD_LIBRARY_PATH' in os.environ:
-                os.environ['LD_LIBRARY_PATH'] = f"{os.path.join(conda_prefix, 'lib')}:{os.environ['LD_LIBRARY_PATH']}"
+            # 1. Adjust grid spacing if necessary
+            dx = self.params['dx']
+            if dx >= self.params['element_width']:
+                dx = self.params['element_width'] / 2
+                Nx = int(round((self.params['Xrange'][1] - self.params['Xrange'][0]) / dx))
+                Nz = int(round((self.params['Zrange'][1] - self.params['Zrange'][0]) / dx))
             else:
-                os.environ['LD_LIBRARY_PATH'] = os.path.join(conda_prefix, 'lib')
+                Nx = self.params['Nx']
+                Nz = self.params['Nz']
 
-            # Load the library globally to make it available for the current Python process
-            try:
-                ctypes.CDLL(libsz_path, mode=ctypes.RTLD_GLOBAL)
-                print(f"Successfully loaded libsz.so.2 from {libsz_path}")
-                print(f"Updated LD_LIBRARY_PATH: {os.environ['LD_LIBRARY_PATH']}")
-            except OSError as e:
-                raise RuntimeError(f"Failed to load libsz.so.2 from {libsz_path}. Install it with: conda install -c conda-forge libaec")
+            # 2. Calculate element positions and select active elements
+            element_positions = np.linspace(
+                self.params['Xrange'][0] + self.params['element_width'] / 2,
+                self.params['Xrange'][1] - self.params['element_width'] / 2,
+                self.params['num_elements']
+            )
+            center_idx = np.argmin(np.abs(element_positions - self.focal_line))
+            start_idx = max(0, center_idx - self.params['N_piezoFocal'] // 2)
+            end_idx = min(self.params['num_elements'] - 1, start_idx + self.params['N_piezoFocal'] - 1)
+            selected_indices = np.arange(start_idx, end_idx + 1)
+            num_active_elements = len(selected_indices)  # Nombre d'éléments actifs
 
+            # 3. Calculate grid factors for downsampling
+            factorT = int(np.ceil(self.params['f_AQ'] / self.params['f_saving']))
+            factorX = int(np.ceil(Nx / self.params['Nx']))
+            factorZ = int(np.ceil(Nz / self.params['Nz']))
 
-            # Create a source mask for the transducer
+            # 4. Create source mask
             source = kSource()
-            source.p_mask = np.zeros((self.params['Nx'], self.params['Nz']))
-            source.p = np.zeros((self.params['num_elements'], self.delayedSignal.shape[1]))  # Initialize source pressure
+            source.p_mask = np.zeros((Nx, Nz))
+            kgrid = kWaveGrid([Nx, Nz], [dx, dx])
+            kgrid.setTime(self.kgrid.Nt, 1 / self.params['f_AQ'])
 
-            # Calculate the center of the transducer
-            center_index = self.params['Nx'] // 2
-            coeff = self.delayedSignal.shape[0] // self.params['num_elements']
-
-            if self.delayedSignal.shape[0] % self.params['num_elements'] != 0:
-                raise ValueError("The number of elements must be a divisor of the delayed signal length.")
-
-            # Set the active elements in the source mask
-            element_width_grid_points = int(round(self.params['element_width'] / self.params['dx']))
+            # 5. Calculate element width in grid points
+            element_width_grid_points = int(round(self.params['element_width'] / dx))
+            current_position = (Nx - (self.params['num_elements'] * element_width_grid_points)) // 2
 
+            # 6. Set active elements in source mask
             for i in range(self.params['num_elements']):
-                source.p[i] = self.delayedSignal[i * coeff]
-                x_pos = center_index - (self.params['num_elements'] // 2) * element_width_grid_points + i * element_width_grid_points
-                source.p_mask[x_pos, 0] = 1
+                if i in selected_indices:
+                    x_start = current_position
+                    x_end = current_position + element_width_grid_points
+                    source.p_mask[x_start:x_end, 0] = 1  # Marque tous les points de grille de l'élément
+                current_position += element_width_grid_points
 
-            # Define sensors to observe acoustic fields
+            # 7. Prepare delayed signals (un signal par élément actif)
+            if factorT != 1:
+                delayedSignal = self._apply_delay(dx=dx)  # Doit retourner (num_elements, ...)
+            else:
+                delayedSignal = self.delayedSignal
+
+            # 8. Sélectionner les signaux pour les éléments actifs uniquement
+            source.p = delayedSignal[selected_indices, :]  # Un signal par élément actif
+
+            # 9. Define sensor mask
             sensor = kSensor()
-            sensor.mask = np.ones((self.params['Nx'], self.params['Nz']))
+            sensor.mask = np.ones((Nx, Nz))
 
-            # Simulation options
+            # 10. Calculate PML sizes
+            total_size_x = next_power_of_2(Nx)
+            total_size_z = next_power_of_2(Nz)
+            pml_x_size = (total_size_x - Nx) // 2
+            pml_z_size = (total_size_z - Nz) // 2
+
+            # 11. Simulation options
             simulation_options = SimulationOptions(
                 pml_inside=False,
-                pml_x_size=20,
-                pml_z_size=20,
+                pml_size=[pml_x_size, pml_z_size],
                 use_sg=False,
                 save_to_disk=True,
                 input_filename=os.path.join(gettempdir(), "KwaveIN.h5"),
@@ -187,10 +201,9 @@ class FocusedWave(AcousticField):
                 show_sim_log=show_log
             )
 
-            # Run the simulation
-            print("Starting simulation...")
+            # 12. Run simulation
             sensor_data = kspaceFirstOrder2D(
-                kgrid=self.kgrid,
+                kgrid=kgrid,
                 medium=self.medium,
                 source=source,
                 sensor=sensor,
@@ -198,14 +211,19 @@ class FocusedWave(AcousticField):
                 execution_options=execution_options,
             )
 
-            print("Simulation completed successfully.")
-            return sensor_data['p'].reshape(self.kgrid.Nt, self.params['Nz'], self.params['Nx'])
+            # 13. Process results
+            data = sensor_data['p'].reshape(kgrid.Nt, Nz, Nx)
+            if factorT != 1 or factorX != 1 or factorZ != 1:
+                return reshape_field(data, [factorT, factorX, factorZ])
+            else:
+                return data
 
         except Exception as e:
             print(f"Error generating 2D acoustic field: {e}")
             return None
 
 
+
     def _generate_3Dacoustic_field_KWAVE(self, isGPU=True if config.get_process() == 'gpu' else False, show_log = True):
         """
         Generate a 3D acoustic field using k-Wave simulation for a focused wave.
